@@ -155,6 +155,7 @@ mod tests {
     use crate::syntax::Text;
     use crate::syntax::TextUnit;
     use crate::syntax::WordUnit;
+    use assert_matches::assert_matches;
     use futures_executor::block_on;
 
     #[test]
@@ -192,15 +193,13 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(switch.r#type, SwitchType::Alter);
             assert_eq!(switch.condition, SwitchCondition::Unset);
             assert_eq!(switch.word.units, []);
-            assert_eq!(switch.word.location.line.value, "+}");
-            assert_eq!(switch.word.location.column.get(), 2);
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+            assert_eq!(*switch.word.location.code.value.borrow(), "+}");
+            assert_eq!(switch.word.location.index, 1);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -214,7 +213,7 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(switch.r#type, SwitchType::Alter);
             assert_eq!(switch.condition, SwitchCondition::Unset);
             assert_eq!(
@@ -226,11 +225,9 @@ mod tests {
                     WordUnit::Unquoted(TextUnit::Literal('z')),
                 ]
             );
-            assert_eq!(switch.word.location.line.value, "+a  z}");
-            assert_eq!(switch.word.location.column.get(), 2);
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+            assert_eq!(*switch.word.location.code.value.borrow(), "+a  z}");
+            assert_eq!(switch.word.location.index, 1);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -244,15 +241,13 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(switch.r#type, SwitchType::Alter);
             assert_eq!(switch.condition, SwitchCondition::UnsetOrEmpty);
             assert_eq!(switch.word.units, []);
-            assert_eq!(switch.word.location.line.value, ":+}");
-            assert_eq!(switch.word.location.column.get(), 3);
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+            assert_eq!(*switch.word.location.code.value.borrow(), ":+}");
+            assert_eq!(switch.word.location.index, 2);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -266,15 +261,13 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(switch.r#type, SwitchType::Default);
             assert_eq!(switch.condition, SwitchCondition::Unset);
             assert_eq!(switch.word.units, []);
-            assert_eq!(switch.word.location.line.value, "-}");
-            assert_eq!(switch.word.location.column.get(), 2);
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+            assert_eq!(*switch.word.location.code.value.borrow(), "-}");
+            assert_eq!(switch.word.location.index, 1);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -288,7 +281,7 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(switch.r#type, SwitchType::Default);
             assert_eq!(switch.condition, SwitchCondition::UnsetOrEmpty);
             assert_eq!(
@@ -300,11 +293,9 @@ mod tests {
                     WordUnit::Unquoted(TextUnit::Literal('l')),
                 ]
             );
-            assert_eq!(switch.word.location.line.value, ":-cool}");
-            assert_eq!(switch.word.location.column.get(), 3);
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+            assert_eq!(*switch.word.location.code.value.borrow(), ":-cool}");
+            assert_eq!(switch.word.location.index, 2);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -318,15 +309,13 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(switch.r#type, SwitchType::Assign);
             assert_eq!(switch.condition, SwitchCondition::UnsetOrEmpty);
             assert_eq!(switch.word.units, []);
-            assert_eq!(switch.word.location.line.value, ":=}");
-            assert_eq!(switch.word.location.column.get(), 3);
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+            assert_eq!(*switch.word.location.code.value.borrow(), ":=}");
+            assert_eq!(switch.word.location.index, 2);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -340,7 +329,7 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(switch.r#type, SwitchType::Assign);
             assert_eq!(switch.condition, SwitchCondition::Unset);
             assert_eq!(
@@ -351,11 +340,9 @@ mod tests {
                     WordUnit::Unquoted(TextUnit::Literal('s')),
                 ]
             );
-            assert_eq!(switch.word.location.line.value, "=Yes}");
-            assert_eq!(switch.word.location.column.get(), 2);
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+            assert_eq!(*switch.word.location.code.value.borrow(), "=Yes}");
+            assert_eq!(switch.word.location.index, 1);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -369,15 +356,13 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(switch.r#type, SwitchType::Error);
             assert_eq!(switch.condition, SwitchCondition::Unset);
             assert_eq!(switch.word.units, []);
-            assert_eq!(switch.word.location.line.value, "?}");
-            assert_eq!(switch.word.location.column.get(), 2);
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+            assert_eq!(*switch.word.location.code.value.borrow(), "?}");
+            assert_eq!(switch.word.location.index, 1);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -391,7 +376,7 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(switch.r#type, SwitchType::Error);
             assert_eq!(switch.condition, SwitchCondition::UnsetOrEmpty);
             assert_eq!(
@@ -401,11 +386,9 @@ mod tests {
                     WordUnit::Unquoted(TextUnit::Literal('o')),
                 ]
             );
-            assert_eq!(switch.word.location.line.value, ":?No}");
-            assert_eq!(switch.word.location.column.get(), 3);
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+            assert_eq!(*switch.word.location.code.value.borrow(), ":?No}");
+            assert_eq!(switch.word.location.index, 2);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -419,11 +402,9 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(switch.word.units, [WordUnit::Tilde("".to_string())]);
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+        });
     }
 
     #[test]
@@ -435,14 +416,12 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Switch(switch) = result {
+        assert_matches!(result, Modifier::Switch(switch) => {
             assert_eq!(
                 switch.word.units,
                 [WordUnit::Unquoted(TextUnit::Literal('~'))]
             );
-        } else {
-            panic!("Not a switch: {:?}", result);
-        }
+        });
     }
 
     #[test]
@@ -454,15 +433,13 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Trim(trim) = result {
+        assert_matches!(result, Modifier::Trim(trim) => {
             assert_eq!(trim.side, TrimSide::Prefix);
             assert_eq!(trim.length, TrimLength::Shortest);
             assert_eq!(trim.pattern.units, [WordUnit::SingleQuote("*".to_string())]);
-            assert_eq!(trim.pattern.location.line.value, "#'*'}");
-            assert_eq!(trim.pattern.location.column.get(), 2);
-        } else {
-            panic!("Not a trim: {:?}", result);
-        }
+            assert_eq!(*trim.pattern.location.code.value.borrow(), "#'*'}");
+            assert_eq!(trim.pattern.location.index, 1);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -476,15 +453,13 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Trim(trim) = result {
+        assert_matches!(result, Modifier::Trim(trim) => {
             assert_eq!(trim.side, TrimSide::Prefix);
             assert_eq!(trim.length, TrimLength::Shortest);
             assert_eq!(trim.pattern.units, [WordUnit::SingleQuote("*".to_string())]);
-            assert_eq!(trim.pattern.location.line.value, "#'*'}");
-            assert_eq!(trim.pattern.location.column.get(), 2);
-        } else {
-            panic!("Not a trim: {:?}", result);
-        }
+            assert_eq!(*trim.pattern.location.code.value.borrow(), "#'*'}");
+            assert_eq!(trim.pattern.location.index, 1);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -498,20 +473,16 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Trim(trim) = result {
+        assert_matches!(result, Modifier::Trim(trim) => {
             assert_eq!(trim.side, TrimSide::Prefix);
             assert_eq!(trim.length, TrimLength::Longest);
             assert_eq!(trim.pattern.units.len(), 1, "{:?}", trim.pattern);
-            if let WordUnit::DoubleQuote(Text(units)) = &trim.pattern.units[0] {
+            assert_matches!(&trim.pattern.units[0], WordUnit::DoubleQuote(Text(units)) => {
                 assert_eq!(units[..], [TextUnit::Literal('?')]);
-            } else {
-                panic!("Not a double quote: {:?}", trim.pattern);
-            }
-            assert_eq!(trim.pattern.location.line.value, r#"##"?"}"#);
-            assert_eq!(trim.pattern.location.column.get(), 3);
-        } else {
-            panic!("Not a trim: {:?}", result);
-        }
+            });
+            assert_eq!(*trim.pattern.location.code.value.borrow(), r#"##"?"}"#);
+            assert_eq!(trim.pattern.location.index, 2);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -525,18 +496,16 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Trim(trim) = result {
+        assert_matches!(result, Modifier::Trim(trim) => {
             assert_eq!(trim.side, TrimSide::Suffix);
             assert_eq!(trim.length, TrimLength::Shortest);
             assert_eq!(
                 trim.pattern.units,
                 [WordUnit::Unquoted(TextUnit::Backslashed('%'))]
             );
-            assert_eq!(trim.pattern.location.line.value, r"%\%}");
-            assert_eq!(trim.pattern.location.column.get(), 2);
-        } else {
-            panic!("Not a trim: {:?}", result);
-        }
+            assert_eq!(*trim.pattern.location.code.value.borrow(), r"%\%}");
+            assert_eq!(trim.pattern.location.index, 1);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -550,18 +519,16 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Trim(trim) = result {
+        assert_matches!(result, Modifier::Trim(trim) => {
             assert_eq!(trim.side, TrimSide::Suffix);
             assert_eq!(trim.length, TrimLength::Longest);
             assert_eq!(
                 trim.pattern.units,
                 [WordUnit::Unquoted(TextUnit::Literal('%'))]
             );
-            assert_eq!(trim.pattern.location.line.value, "%%%}");
-            assert_eq!(trim.pattern.location.column.get(), 3);
-        } else {
-            panic!("Not a trim: {:?}", result);
-        }
+            assert_eq!(*trim.pattern.location.code.value.borrow(), "%%%}");
+            assert_eq!(trim.pattern.location.index, 2);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(Some('}')));
     }
@@ -575,11 +542,9 @@ mod tests {
         };
 
         let result = block_on(lexer.suffix_modifier()).unwrap();
-        if let Modifier::Trim(trim) = result {
+        assert_matches!(result, Modifier::Trim(trim) => {
             assert_eq!(trim.pattern.units, [WordUnit::Tilde("".to_string())]);
-        } else {
-            panic!("Not a trim: {:?}", result);
-        }
+        });
     }
 
     #[test]
@@ -592,8 +557,8 @@ mod tests {
 
         let e = block_on(lexer.suffix_modifier()).unwrap_err();
         assert_eq!(e.cause, ErrorCause::Syntax(SyntaxError::InvalidModifier));
-        assert_eq!(e.location.line.value, ":");
-        assert_eq!(e.location.column.get(), 2);
+        assert_eq!(*e.location.code.value.borrow(), ":");
+        assert_eq!(e.location.index, 1);
     }
 
     #[test]
@@ -606,8 +571,8 @@ mod tests {
 
         let e = block_on(lexer.suffix_modifier()).unwrap_err();
         assert_eq!(e.cause, ErrorCause::Syntax(SyntaxError::InvalidModifier));
-        assert_eq!(e.location.line.value, ":x}");
-        assert_eq!(e.location.column.get(), 2);
+        assert_eq!(*e.location.code.value.borrow(), ":x}");
+        assert_eq!(e.location.index, 1);
     }
 
     #[test]
@@ -620,7 +585,7 @@ mod tests {
 
         let e = block_on(lexer.suffix_modifier()).unwrap_err();
         assert_eq!(e.cause, ErrorCause::Syntax(SyntaxError::InvalidModifier));
-        assert_eq!(e.location.line.value, ":#}");
-        assert_eq!(e.location.column.get(), 2);
+        assert_eq!(*e.location.code.value.borrow(), ":#}");
+        assert_eq!(e.location.index, 1);
     }
 }
