@@ -221,6 +221,7 @@ mod tests {
     use crate::syntax::Backquote;
     use crate::syntax::BackquoteUnit;
     use crate::syntax::CommandSubst;
+    use assert_matches::assert_matches;
     use futures_executor::block_on;
 
     #[test]
@@ -237,16 +238,12 @@ mod tests {
                 assert_eq!(c, 'X');
                 false
             },
-            |c| panic!("unexpected call to is_escapable({:?})", c),
+            |c| unreachable!("unexpected call to is_escapable({:?})", c),
         ))
         .unwrap()
         .unwrap();
         assert!(called);
-        if let Literal(c) = result {
-            assert_eq!(c, 'X');
-        } else {
-            panic!("unexpected result {:?}", result);
-        }
+        assert_matches!(result, Literal('X'));
 
         assert_eq!(block_on(lexer.peek_char()), Ok(None));
     }
@@ -265,7 +262,7 @@ mod tests {
                 assert_eq!(c, ';');
                 true
             },
-            |c| panic!("unexpected call to is_escapable({:?})", c),
+            |c| unreachable!("unexpected call to is_escapable({:?})", c),
         ))
         .unwrap();
         assert!(called);
@@ -283,7 +280,7 @@ mod tests {
         };
         let mut called = false;
         let result = block_on(lexer.text_unit(
-            |c| panic!("unexpected call to is_delimiter({:?})", c),
+            |c| unreachable!("unexpected call to is_delimiter({:?})", c),
             |c| {
                 called = true;
                 assert_eq!(c, '#');
@@ -306,8 +303,8 @@ mod tests {
             context: WordContext::Word,
         };
         let result = block_on(lexer.text_unit(
-            |c| panic!("unexpected call to is_delimiter({:?})", c),
-            |c| panic!("unexpected call to is_escapable({:?})", c),
+            |c| unreachable!("unexpected call to is_delimiter({:?})", c),
+            |c| unreachable!("unexpected call to is_escapable({:?})", c),
         ))
         .unwrap()
         .unwrap();
@@ -325,7 +322,7 @@ mod tests {
         };
         let mut called = false;
         let result = block_on(lexer.text_unit(
-            |c| panic!("unexpected call to is_delimiter({:?})", c),
+            |c| unreachable!("unexpected call to is_delimiter({:?})", c),
             |c| {
                 called = true;
                 assert_eq!(c, '\\');
@@ -348,17 +345,15 @@ mod tests {
             context: WordContext::Word,
         };
         let result = block_on(lexer.text_unit(
-            |c| panic!("unexpected call to is_delimiter({:?})", c),
-            |c| panic!("unexpected call to is_escapable({:?})", c),
+            |c| unreachable!("unexpected call to is_delimiter({:?})", c),
+            |c| unreachable!("unexpected call to is_escapable({:?})", c),
         ))
         .unwrap()
         .unwrap();
-        if let CommandSubst { content, location } = result {
+        assert_matches!(result, CommandSubst { content, location } => {
             assert_eq!(content, "");
-            assert_eq!(location.column.get(), 1);
-        } else {
-            panic!("unexpected result {:?}", result);
-        }
+            assert_eq!(location.index, 0);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(None));
     }
@@ -371,17 +366,15 @@ mod tests {
             context: WordContext::Text,
         };
         let result = block_on(lexer.text_unit(
-            |c| panic!("unexpected call to is_delimiter({:?})", c),
-            |c| panic!("unexpected call to is_escapable({:?})", c),
+            |c| unreachable!("unexpected call to is_delimiter({:?})", c),
+            |c| unreachable!("unexpected call to is_escapable({:?})", c),
         ))
         .unwrap()
         .unwrap();
-        if let Backquote { content, location } = result {
+        assert_matches!(result, Backquote { content, location } => {
             assert_eq!(content, [BackquoteUnit::Backslashed('"')]);
-            assert_eq!(location.column.get(), 1);
-        } else {
-            panic!("Not a backquote: {:?}", result);
-        }
+            assert_eq!(location.index, 0);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(None));
     }
@@ -394,20 +387,18 @@ mod tests {
             context: WordContext::Word,
         };
         let result = block_on(lexer.text_unit(
-            |c| panic!("unexpected call to is_delimiter({:?})", c),
-            |c| panic!("unexpected call to is_escapable({:?})", c),
+            |c| unreachable!("unexpected call to is_delimiter({:?})", c),
+            |c| unreachable!("unexpected call to is_escapable({:?})", c),
         ))
         .unwrap()
         .unwrap();
-        if let Backquote { content, location } = result {
+        assert_matches!(result, Backquote { content, location } => {
             assert_eq!(
                 content,
                 [BackquoteUnit::Literal('\\'), BackquoteUnit::Literal('"')]
             );
-            assert_eq!(location.column.get(), 1);
-        } else {
-            panic!("Not a backquote: {:?}", result);
-        }
+            assert_eq!(location.index, 0);
+        });
 
         assert_eq!(block_on(lexer.peek_char()), Ok(None));
     }
@@ -421,15 +412,11 @@ mod tests {
         };
         let result = block_on(lexer.text_unit(
             |_| false,
-            |c| panic!("unexpected call to is_escapable({:?})", c),
+            |c| unreachable!("unexpected call to is_escapable({:?})", c),
         ))
         .unwrap()
         .unwrap();
-        if let Literal(c) = result {
-            assert_eq!(c, 'X');
-        } else {
-            panic!("unexpected result {:?}", result);
-        }
+        assert_eq!(result, Literal('X'));
 
         assert_eq!(block_on(lexer.peek_char()), Ok(None));
     }
@@ -438,8 +425,8 @@ mod tests {
     fn lexer_text_empty() {
         let mut lexer = Lexer::from_memory("", Source::Unknown);
         let Text(units) = block_on(lexer.text(
-            |c| panic!("unexpected call to is_delimiter({:?})", c),
-            |c| panic!("unexpected call to is_escapable({:?})", c),
+            |c| unreachable!("unexpected call to is_delimiter({:?})", c),
+            |c| unreachable!("unexpected call to is_escapable({:?})", c),
         ))
         .unwrap();
         assert_eq!(units, &[]);
@@ -460,7 +447,7 @@ mod tests {
                 called += 1;
                 false
             },
-            |c| panic!("unexpected call to is_escapable({:?})", c),
+            |c| unreachable!("unexpected call to is_escapable({:?})", c),
         ))
         .unwrap();
         assert_eq!(units, &[Literal('a'), Literal('b'), Literal('c')]);
@@ -484,7 +471,7 @@ mod tests {
                 called += 1;
                 c == 'c'
             },
-            |c| panic!("unexpected call to is_escapable({:?})", c),
+            |c| unreachable!("unexpected call to is_escapable({:?})", c),
         ))
         .unwrap();
         assert_eq!(units, &[Literal('a'), Literal('b')]);
@@ -603,17 +590,16 @@ mod tests {
     fn lexer_text_with_parentheses_unclosed() {
         let mut lexer = Lexer::from_memory("x(()", Source::Unknown);
         let e = block_on(lexer.text_with_parentheses(|_| false, |_| false)).unwrap_err();
-        if let ErrorCause::Syntax(SyntaxError::UnclosedParen { opening_location }) = e.cause {
-            assert_eq!(opening_location.line.value, "x(()");
-            assert_eq!(opening_location.line.number.get(), 1);
-            assert_eq!(opening_location.line.source, Source::Unknown);
-            assert_eq!(opening_location.column.get(), 2);
-        } else {
-            panic!("unexpected error cause {:?}", e);
-        }
-        assert_eq!(e.location.line.value, "x(()");
-        assert_eq!(e.location.line.number.get(), 1);
-        assert_eq!(e.location.line.source, Source::Unknown);
-        assert_eq!(e.location.column.get(), 5);
+        assert_matches!(e.cause,
+            ErrorCause::Syntax(SyntaxError::UnclosedParen { opening_location }) => {
+            assert_eq!(*opening_location.code.value.borrow(), "x(()");
+            assert_eq!(opening_location.code.start_line_number.get(), 1);
+            assert_eq!(opening_location.code.source, Source::Unknown);
+            assert_eq!(opening_location.index, 1);
+        });
+        assert_eq!(*e.location.code.value.borrow(), "x(()");
+        assert_eq!(e.location.code.start_line_number.get(), 1);
+        assert_eq!(e.location.code.source, Source::Unknown);
+        assert_eq!(e.location.index, 4);
     }
 }
