@@ -171,6 +171,7 @@ mod tests {
     use crate::source::Source;
     use crate::syntax::RedirBody;
     use crate::syntax::RedirOp;
+    use assert_matches::assert_matches;
     use futures_executor::block_on;
 
     #[test]
@@ -230,18 +231,17 @@ mod tests {
         let aliases = Default::default();
         let mut parser = Parser::new(&mut lexer, &aliases);
         let e = block_on(parser.array_values()).unwrap_err();
-        if let ErrorCause::Syntax(SyntaxError::UnclosedArrayValue { opening_location }) = e.cause {
-            assert_eq!(opening_location.line.value, "(a b");
-            assert_eq!(opening_location.line.number.get(), 1);
-            assert_eq!(opening_location.line.source, Source::Unknown);
-            assert_eq!(opening_location.column.get(), 1);
-        } else {
-            panic!("Unexpected cause {:?}", e.cause);
-        }
-        assert_eq!(e.location.line.value, "(a b");
-        assert_eq!(e.location.line.number.get(), 1);
-        assert_eq!(e.location.line.source, Source::Unknown);
-        assert_eq!(e.location.column.get(), 5);
+        assert_matches!(e.cause,
+             ErrorCause::Syntax(SyntaxError::UnclosedArrayValue { opening_location }) => {
+            assert_eq!(*opening_location.code.value.borrow(), "(a b");
+            assert_eq!(opening_location.code.start_line_number.get(), 1);
+            assert_eq!(opening_location.code.source, Source::Unknown);
+            assert_eq!(opening_location.index, 0);
+        });
+        assert_eq!(*e.location.code.value.borrow(), "(a b");
+        assert_eq!(e.location.code.start_line_number.get(), 1);
+        assert_eq!(e.location.code.source, Source::Unknown);
+        assert_eq!(e.location.index, 4);
     }
 
     #[test]
@@ -250,18 +250,17 @@ mod tests {
         let aliases = Default::default();
         let mut parser = Parser::new(&mut lexer, &aliases);
         let e = block_on(parser.array_values()).unwrap_err();
-        if let ErrorCause::Syntax(SyntaxError::UnclosedArrayValue { opening_location }) = e.cause {
-            assert_eq!(opening_location.line.value, "(a;b)");
-            assert_eq!(opening_location.line.number.get(), 1);
-            assert_eq!(opening_location.line.source, Source::Unknown);
-            assert_eq!(opening_location.column.get(), 1);
-        } else {
-            panic!("Unexpected cause {:?}", e.cause);
-        }
-        assert_eq!(e.location.line.value, "(a;b)");
-        assert_eq!(e.location.line.number.get(), 1);
-        assert_eq!(e.location.line.source, Source::Unknown);
-        assert_eq!(e.location.column.get(), 3);
+        assert_matches!(e.cause,
+            ErrorCause::Syntax(SyntaxError::UnclosedArrayValue { opening_location }) => {
+            assert_eq!(*opening_location.code.value.borrow(), "(a;b)");
+            assert_eq!(opening_location.code.start_line_number.get(), 1);
+            assert_eq!(opening_location.code.source, Source::Unknown);
+            assert_eq!(opening_location.index, 0);
+        });
+        assert_eq!(*e.location.code.value.borrow(), "(a;b)");
+        assert_eq!(e.location.code.start_line_number.get(), 1);
+        assert_eq!(e.location.code.source, Source::Unknown);
+        assert_eq!(e.location.index, 2);
     }
 
     #[test]
@@ -296,10 +295,10 @@ mod tests {
         assert_eq!(sc.assigns.len(), 1);
         assert_eq!(sc.assigns[0].name, "my");
         assert_eq!(sc.assigns[0].value.to_string(), "assignment");
-        assert_eq!(sc.assigns[0].location.line.value, "my=assignment");
-        assert_eq!(sc.assigns[0].location.line.number.get(), 1);
-        assert_eq!(sc.assigns[0].location.line.source, Source::Unknown);
-        assert_eq!(sc.assigns[0].location.column.get(), 1);
+        assert_eq!(*sc.assigns[0].location.code.value.borrow(), "my=assignment");
+        assert_eq!(sc.assigns[0].location.code.start_line_number.get(), 1);
+        assert_eq!(sc.assigns[0].location.code.source, Source::Unknown);
+        assert_eq!(sc.assigns[0].location.index, 0);
     }
 
     #[test]
@@ -314,22 +313,22 @@ mod tests {
         assert_eq!(sc.assigns.len(), 3);
         assert_eq!(sc.assigns[0].name, "a");
         assert_eq!(sc.assigns[0].value.to_string(), "");
-        assert_eq!(sc.assigns[0].location.line.value, "a= b=! c=X");
-        assert_eq!(sc.assigns[0].location.line.number.get(), 1);
-        assert_eq!(sc.assigns[0].location.line.source, Source::Unknown);
-        assert_eq!(sc.assigns[0].location.column.get(), 1);
+        assert_eq!(*sc.assigns[0].location.code.value.borrow(), "a= b=! c=X");
+        assert_eq!(sc.assigns[0].location.code.start_line_number.get(), 1);
+        assert_eq!(sc.assigns[0].location.code.source, Source::Unknown);
+        assert_eq!(sc.assigns[0].location.index, 0);
         assert_eq!(sc.assigns[1].name, "b");
         assert_eq!(sc.assigns[1].value.to_string(), "!");
-        assert_eq!(sc.assigns[1].location.line.value, "a= b=! c=X");
-        assert_eq!(sc.assigns[1].location.line.number.get(), 1);
-        assert_eq!(sc.assigns[1].location.line.source, Source::Unknown);
-        assert_eq!(sc.assigns[1].location.column.get(), 4);
+        assert_eq!(*sc.assigns[1].location.code.value.borrow(), "a= b=! c=X");
+        assert_eq!(sc.assigns[1].location.code.start_line_number.get(), 1);
+        assert_eq!(sc.assigns[1].location.code.source, Source::Unknown);
+        assert_eq!(sc.assigns[1].location.index, 3);
         assert_eq!(sc.assigns[2].name, "c");
         assert_eq!(sc.assigns[2].value.to_string(), "X");
-        assert_eq!(sc.assigns[2].location.line.value, "a= b=! c=X");
-        assert_eq!(sc.assigns[2].location.line.number.get(), 1);
-        assert_eq!(sc.assigns[2].location.line.source, Source::Unknown);
-        assert_eq!(sc.assigns[2].location.column.get(), 8);
+        assert_eq!(*sc.assigns[2].location.code.value.borrow(), "a= b=! c=X");
+        assert_eq!(sc.assigns[2].location.code.start_line_number.get(), 1);
+        assert_eq!(sc.assigns[2].location.code.source, Source::Unknown);
+        assert_eq!(sc.assigns[2].location.index, 7);
     }
 
     #[test]
@@ -371,16 +370,10 @@ mod tests {
         assert_eq!(sc.words, []);
         assert_eq!(sc.redirs.len(), 1);
         assert_eq!(sc.redirs[0].fd, None);
-        if let RedirBody::Normal {
-            ref operator,
-            ref operand,
-        } = sc.redirs[0].body
-        {
+        assert_matches!(sc.redirs[0].body, RedirBody::Normal { ref operator, ref operand } => {
             assert_eq!(operator, &RedirOp::FileIn);
             assert_eq!(operand.to_string(), "foo")
-        } else {
-            panic!("Unexpected redirection body {:?}", sc.redirs[0].body);
-        }
+        });
 
         let next = block_on(parser.peek_token()).unwrap();
         assert_eq!(next.id, EndOfInput);
@@ -397,38 +390,20 @@ mod tests {
         assert_eq!(sc.words, []);
         assert_eq!(sc.redirs.len(), 3);
         assert_eq!(sc.redirs[0].fd, None);
-        if let RedirBody::Normal {
-            ref operator,
-            ref operand,
-        } = sc.redirs[0].body
-        {
+        assert_matches!(sc.redirs[0].body, RedirBody::Normal { ref operator, ref operand } => {
             assert_eq!(operator, &RedirOp::FileIn);
             assert_eq!(operand.to_string(), "one")
-        } else {
-            panic!("Unexpected redirection body {:?}", sc.redirs[0].body);
-        }
+        });
         assert_eq!(sc.redirs[1].fd, None);
-        if let RedirBody::Normal {
-            ref operator,
-            ref operand,
-        } = sc.redirs[1].body
-        {
+        assert_matches!(sc.redirs[1].body, RedirBody::Normal { ref operator, ref operand } => {
             assert_eq!(operator, &RedirOp::FileOut);
             assert_eq!(operand.to_string(), "two")
-        } else {
-            panic!("Unexpected redirection body {:?}", sc.redirs[1].body);
-        }
+        });
         assert_eq!(sc.redirs[2].fd, None);
-        if let RedirBody::Normal {
-            ref operator,
-            ref operand,
-        } = sc.redirs[2].body
-        {
+        assert_matches!(sc.redirs[2].body, RedirBody::Normal { ref operator, ref operand } => {
             assert_eq!(operator, &RedirOp::FileAppend);
             assert_eq!(operand.to_string(), "three")
-        } else {
-            panic!("Unexpected redirection body {:?}", sc.redirs[2].body);
-        }
+        });
 
         let next = block_on(parser.peek_token()).unwrap();
         assert_eq!(next.id, EndOfInput);
@@ -461,16 +436,10 @@ mod tests {
         assert_eq!(sc.redirs.len(), 1);
         assert_eq!(sc.words[0].to_string(), "word");
         assert_eq!(sc.redirs[0].fd, None);
-        if let RedirBody::Normal {
-            ref operator,
-            ref operand,
-        } = sc.redirs[0].body
-        {
+        assert_matches!(sc.redirs[0].body, RedirBody::Normal { ref operator, ref operand } => {
             assert_eq!(operator, &RedirOp::FileIn);
             assert_eq!(operand.to_string(), "redirection")
-        } else {
-            panic!("Unexpected redirection body {:?}", sc.redirs[0].body);
-        }
+        });
     }
 
     #[test]
@@ -486,16 +455,10 @@ mod tests {
         assert_eq!(sc.assigns[0].name, "a");
         assert_eq!(sc.assigns[0].value.to_string(), "b");
         assert_eq!(sc.redirs[0].fd, None);
-        if let RedirBody::Normal {
-            ref operator,
-            ref operand,
-        } = sc.redirs[0].body
-        {
+        assert_matches!(sc.redirs[0].body, RedirBody::Normal { ref operator, ref operand } => {
             assert_eq!(operator, &RedirOp::FileIn);
             assert_eq!(operand.to_string(), "foo")
-        } else {
-            panic!("Unexpected redirection body {:?}", sc.redirs[0].body);
-        }
+        });
     }
 
     #[test]
@@ -512,16 +475,10 @@ mod tests {
         assert_eq!(sc.assigns[0].value.to_string(), "then");
         assert_eq!(sc.words[0].to_string(), "else");
         assert_eq!(sc.redirs[0].fd, None);
-        if let RedirBody::Normal {
-            ref operator,
-            ref operand,
-        } = sc.redirs[0].body
-        {
+        assert_matches!(sc.redirs[0].body, RedirBody::Normal { ref operator, ref operand } => {
             assert_eq!(operator, &RedirOp::FileIn);
             assert_eq!(operand.to_string(), "foo")
-        } else {
-            panic!("Unexpected redirection body {:?}", sc.redirs[0].body);
-        }
+        });
     }
 
     #[test]
@@ -535,11 +492,9 @@ mod tests {
         assert_eq!(sc.words, []);
         assert_eq!(*sc.redirs, []);
         assert_eq!(sc.assigns[0].name, "a");
-        if let Array(words) = &sc.assigns[0].value {
+        assert_matches!(&sc.assigns[0].value, Array(words) => {
             assert_eq!(words, &[]);
-        } else {
-            panic!("Non-array assignment value {:?}", sc.assigns[0].value);
-        }
+        });
 
         let next = block_on(parser.peek_token()).unwrap();
         assert_eq!(next.id, EndOfInput);
@@ -557,10 +512,10 @@ mod tests {
         assert_eq!(*sc.redirs, []);
         assert_eq!(sc.assigns[0].name, "a");
         assert_eq!(sc.assigns[0].value.to_string(), "");
-        assert_eq!(sc.assigns[0].location.line.value, "a= ()");
-        assert_eq!(sc.assigns[0].location.line.number.get(), 1);
-        assert_eq!(sc.assigns[0].location.line.source, Source::Unknown);
-        assert_eq!(sc.assigns[0].location.column.get(), 1);
+        assert_eq!(*sc.assigns[0].location.code.value.borrow(), "a= ()");
+        assert_eq!(sc.assigns[0].location.code.start_line_number.get(), 1);
+        assert_eq!(sc.assigns[0].location.code.source, Source::Unknown);
+        assert_eq!(sc.assigns[0].location.index, 0);
 
         let next = block_on(parser.peek_token()).unwrap();
         assert_eq!(next.id, Operator(OpenParen));
@@ -578,10 +533,10 @@ mod tests {
         assert_eq!(*sc.redirs, []);
         assert_eq!(sc.assigns[0].name, "a");
         assert_eq!(sc.assigns[0].value.to_string(), "b");
-        assert_eq!(sc.assigns[0].location.line.value, "a=b()");
-        assert_eq!(sc.assigns[0].location.line.number.get(), 1);
-        assert_eq!(sc.assigns[0].location.line.source, Source::Unknown);
-        assert_eq!(sc.assigns[0].location.column.get(), 1);
+        assert_eq!(*sc.assigns[0].location.code.value.borrow(), "a=b()");
+        assert_eq!(sc.assigns[0].location.code.start_line_number.get(), 1);
+        assert_eq!(sc.assigns[0].location.code.source, Source::Unknown);
+        assert_eq!(sc.assigns[0].location.index, 0);
 
         let next = block_on(parser.peek_token()).unwrap();
         assert_eq!(next.id, Operator(OpenParen));
